@@ -1,28 +1,21 @@
-/*
- * vms_args.c -- command line parsing, to emulate shell i/o redirection.
- *		[ Escape sequence parsing now suppressed. ]
- */
+/* vms_args.c -- command line parsing, to emulate shell i/o redirection.
+  		[ Escape sequence parsing now suppressed. ]
 
-/*
- * Copyright (C) 1991-1996 the Free Software Foundation, Inc.
- *
- * This file is part of GAWK, the GNU implementation of the
- * AWK Programming Language.
- *
- * GAWK is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
- *
- * GAWK is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
- */
+   Copyright (C) 1991-1996, 1997 the Free Software Foundation, Inc.
+
+   This program is free software; you can redistribute it and/or modify
+   it under the terms of the GNU General Public License as published by
+   the Free Software Foundation; either version 2, or (at your option)
+   any later version.
+
+   This program is distributed in the hope that it will be useful,
+   but WITHOUT ANY WARRANTY; without even the implied warranty of
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+   GNU General Public License for more details.
+
+   You should have received a copy of the GNU General Public License
+   along with this program; if not, write to the Free Software Foundation,
+   Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */
 
 /*
  * [.vms]vms_arg_fixup - emulate shell's command line processing: handle
@@ -206,7 +199,7 @@ vms_arg_fixup( int *pargc, char ***pargv )
 		    fatal("conflicting specifications for stdout");
 		else {
 		    out_to_err_redirect = 1;
-		    f_out = "SYS$ERROR:";
+		 /* f_out = "SYS$ERROR:"; */
 		}  break;
 	  case '|':		/* pipe */
 	      /* command pipelines are not supported */
